import unittest
import mock
import random

import numpy as np

import bilby.gw.sampler.proposal
from bilby.core import prior
from bilby.core.sampler import proposal


class TestSample(unittest.TestCase):

    def setUp(self):
        self.sample = proposal.Sample(dict(a=1, c=2))

    def tearDown(self):
        del self.sample

    def test_add_sample(self):
        other = proposal.Sample(dict(a=2, c=5))
        expected = proposal.Sample(dict(a=3, c=7))
        self.assertDictEqual(expected, self.sample + other)

    def test_subtract_sample(self):
        other = proposal.Sample(dict(a=2, c=5))
        expected = proposal.Sample(dict(a=-1, c=-3))
        self.assertDictEqual(expected, self.sample - other)

    def test_multiply_sample(self):
        other = 2
        expected = proposal.Sample(dict(a=2, c=4))
        self.assertDictEqual(expected, self.sample * other)


class TestJumpProposal(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(reflective=prior.Uniform(minimum=-0.5, maximum=1, boundary='reflective'),
                                           periodic=prior.Uniform(minimum=-0.5, maximum=1, boundary='periodic'),
                                           default=prior.Uniform(minimum=-0.5, maximum=1)))
        self.sample_above = dict(reflective=1.1, periodic=1.1, default=1.1)
        self.sample_below = dict(reflective=-0.6, periodic=-0.6, default=-0.6)
        self.sample_way_above_case1 = dict(reflective=272, periodic=272, default=272)
        self.sample_way_above_case2 = dict(reflective=270.1, periodic=270.1, default=270.1)
        self.sample_way_below_case1 = dict(reflective=-274, periodic=-274.1, default=-274)
        self.sample_way_below_case2 = dict(reflective=-273.1, periodic=-273.1, default=-273.1)
        self.jump_proposal = proposal.JumpProposal(priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.sample_above
        del self.sample_below
        del self.sample_way_above_case1
        del self.sample_way_above_case2
        del self.sample_way_below_case1
        del self.sample_way_below_case2
        del self.jump_proposal

    def test_set_get_log_j(self):
        self.jump_proposal.log_j = 2.3
        self.assertEqual(2.3, self.jump_proposal.log_j)

    def test_boundary_above_reflective(self):
        new_sample = self.jump_proposal(self.sample_above)
        self.assertAlmostEqual(0.9, new_sample['reflective'])

    def test_boundary_above_periodic(self):
        new_sample = self.jump_proposal(self.sample_above)
        self.assertAlmostEqual(-0.4, new_sample['periodic'])

    def test_boundary_above_default(self):
        new_sample = self.jump_proposal(self.sample_above)
        self.assertAlmostEqual(1.1, new_sample['default'])

    def test_boundary_below_reflective(self):
        new_sample = self.jump_proposal(self.sample_below)
        self.assertAlmostEqual(-0.4, new_sample['reflective'])

    def test_boundary_below_periodic(self):
        new_sample = self.jump_proposal(self.sample_below)
        self.assertAlmostEqual(0.9, new_sample['periodic'])

    def test_boundary_below_default(self):
        new_sample = self.jump_proposal(self.sample_below)
        self.assertAlmostEqual(-0.6, new_sample['default'])

    def test_boundary_way_below_reflective_case1(self):
        new_sample = self.jump_proposal(self.sample_way_below_case1)
        self.assertAlmostEqual(0.0, new_sample['reflective'])

    def test_boundary_way_below_reflective_case2(self):
        new_sample = self.jump_proposal(self.sample_way_below_case2)
        self.assertAlmostEqual(-0.1, new_sample['reflective'])

    def test_boundary_way_below_periodic(self):
        new_sample = self.jump_proposal(self.sample_way_below_case2)
        self.assertAlmostEqual(-0.1, new_sample['periodic'])

    def test_boundary_way_above_reflective_case1(self):
        new_sample = self.jump_proposal(self.sample_way_above_case1)
        self.assertAlmostEqual(0.0, new_sample['reflective'])

    def test_boundary_way_above_reflective_case2(self):
        new_sample = self.jump_proposal(self.sample_way_above_case2)
        self.assertAlmostEqual(0.1, new_sample['reflective'])

    def test_boundary_way_above_periodic(self):
        new_sample = self.jump_proposal(self.sample_way_above_case2)
        self.assertAlmostEqual(0.1, new_sample['periodic'])

    def test_priors(self):
        self.assertEqual(self.priors, self.jump_proposal.priors)


class TestNormJump(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(reflective=prior.Uniform(minimum=-0.5, maximum=1, boundary='periodic'),
                                           periodic=prior.Uniform(minimum=-0.5, maximum=1, boundary='reflective'),
                                           default=prior.Uniform(minimum=-0.5, maximum=1)))
        self.jump_proposal = proposal.NormJump(step_size=3.0, priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_step_size_init(self):
        self.assertEqual(3.0, self.jump_proposal.step_size)

    def test_set_step_size(self):
        self.jump_proposal.step_size = 1.0
        self.assertEqual(1.0, self.jump_proposal.step_size)

    def test_jump_proposal_call(self):
        with mock.patch("numpy.random.normal") as m:
            m.return_value = 0.5
            sample = proposal.Sample(dict(reflective=0.0, periodic=0.0, default=0.0))
            new_sample = self.jump_proposal(sample)
            expected = proposal.Sample(dict(reflective=0.5, periodic=0.5, default=0.5))
            self.assertDictEqual(expected, new_sample)


class TestEnsembleWalk(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(reflective=prior.Uniform(minimum=-0.5, maximum=1, boundary='reflective'),
                                           periodic=prior.Uniform(minimum=-0.5, maximum=1, boundary='periodic'),
                                           default=prior.Uniform(minimum=-0.5, maximum=1)))
        self.jump_proposal = proposal.EnsembleWalk(random_number_generator=random.random,
                                                   n_points=4, priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_n_points_init(self):
        self.assertEqual(4, self.jump_proposal.n_points)

    def test_set_n_points(self):
        self.jump_proposal.n_points = 3
        self.assertEqual(3, self.jump_proposal.n_points)

    def test_random_number_generator_init(self):
        self.assertEqual(random.random, self.jump_proposal.random_number_generator)

    def test_get_center_of_mass(self):
        samples = [proposal.Sample(dict(reflective=0.1*i, periodic=0.1*i, default=0.1*i)) for i in range(3)]
        expected = proposal.Sample(dict(reflective=0.1, periodic=0.1, default=0.1))
        actual = self.jump_proposal.get_center_of_mass(samples)
        for key in samples[0].keys():
            self.assertAlmostEqual(expected[key], actual[key])

    def test_jump_proposal_call(self):
        with mock.patch('random.sample') as m:
            self.jump_proposal.random_number_generator = lambda: 2
            m.return_value = [proposal.Sample(dict(periodic=0.3, reflective=0.3, default=0.3)),
                              proposal.Sample(dict(periodic=0.1, reflective=0.1, default=0.1))]
            sample = proposal.Sample(dict(periodic=0.1, reflective=0.1, default=0.1))
            new_sample = self.jump_proposal(sample, coordinates=None)
            expected = proposal.Sample(dict(periodic=0.1, reflective=0.1, default=0.1))
            for key, value in new_sample.items():
                self.assertAlmostEqual(expected[key], value)


class TestEnsembleEnsembleStretch(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(reflective=prior.Uniform(minimum=-0.5, maximum=1, boundary='reflective'),
                                           periodic=prior.Uniform(minimum=-0.5, maximum=1, boundary='periodic'),
                                           default=prior.Uniform(minimum=-0.5, maximum=1)))
        self.jump_proposal = proposal.EnsembleStretch(scale=3.0, priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_scale_init(self):
        self.assertEqual(3.0, self.jump_proposal.scale)

    def test_set_get_scale(self):
        self.jump_proposal.scale = 5.0
        self.assertEqual(5.0, self.jump_proposal.scale)

    def test_jump_proposal_call(self):
        with mock.patch('random.choice') as m:
            with mock.patch('random.uniform') as n:
                second_sample = proposal.Sample(dict(periodic=0.3, reflective=0.3, default=0.3))
                random_number = 0.5
                m.return_value = second_sample
                n.return_value = random_number
                sample = proposal.Sample(dict(periodic=0.1, reflective=0.1, default=0.1))
                new_sample = self.jump_proposal(sample, coordinates=None)
                coords = 0.3 - 0.2 * np.exp(random_number * np.log(self.jump_proposal.scale))
                expected = proposal.Sample(dict(periodic=coords, reflective=coords, default=coords))
                for key, value in new_sample.items():
                    self.assertAlmostEqual(expected[key], value)

    def test_log_j_after_call(self):
        with mock.patch('random.uniform') as m1:
            with mock.patch('numpy.log') as m2:
                with mock.patch('numpy.exp') as m3:
                    m1.return_value = 1
                    m2.return_value = 1
                    m3.return_value = 1
                    coordinates = [proposal.Sample(dict(periodic=0.3, reflective=0.3, default=0.3)),
                                   proposal.Sample(dict(periodic=0.3, reflective=0.3, default=0.3))]
                    sample = proposal.Sample(dict(periodic=0.2, reflective=0.2, default=0.2))
                    self.jump_proposal(sample=sample,
                                       coordinates=coordinates)
                    self.assertEqual(3, self.jump_proposal.log_j)


class TestDifferentialEvolution(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(reflective=prior.Uniform(minimum=-0.5, maximum=1, boundary='reflective'),
                                           periodic=prior.Uniform(minimum=-0.5, maximum=1, boundary='periodic'),
                                           default=prior.Uniform(minimum=-0.5, maximum=1)))
        self.jump_proposal = proposal.DifferentialEvolution(sigma=1e-3, mu=0.5, priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_mu_init(self):
        self.assertEqual(0.5, self.jump_proposal.mu)

    def test_set_get_mu(self):
        self.jump_proposal.mu = 1
        self.assertEqual(1, self.jump_proposal.mu)

    def test_set_get_sigma(self):
        self.jump_proposal.sigma = 2
        self.assertEqual(2, self.jump_proposal.sigma)

    def test_jump_proposal_call(self):
        with mock.patch('random.sample') as m:
            with mock.patch('random.gauss') as n:
                m.return_value = proposal.Sample(dict(periodic=0.2, reflective=0.2, default=0.2)),\
                                 proposal.Sample(dict(periodic=0.3, reflective=0.3, default=0.3))
                n.return_value = 1
                sample = proposal.Sample(dict(periodic=0.1, reflective=0.1, default=0.1))
                expected = proposal.Sample(dict(periodic=0.2, reflective=0.2, default=0.2))
                new_sample = self.jump_proposal(sample, coordinates=None)
                for key, value in new_sample.items():
                    self.assertAlmostEqual(expected[key], value)


class TestEnsembleEigenVector(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(reflective=prior.Uniform(minimum=-0.5, maximum=1, boundary='reflective'),
                                           periodic=prior.Uniform(minimum=-0.5, maximum=1, boundary='periodic'),
                                           default=prior.Uniform(minimum=-0.5, maximum=1)))
        self.jump_proposal = proposal.EnsembleEigenVector(priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_init_eigen_values(self):
        self.assertIsNone(self.jump_proposal.eigen_values)

    def test_init_eigen_vectors(self):
        self.assertIsNone(self.jump_proposal.eigen_vectors)

    def test_init_covariance(self):
        self.assertIsNone(self.jump_proposal.covariance)

    def test_jump_proposal_update_eigenvectors_none(self):
        self.assertIsNone(self.jump_proposal.update_eigenvectors(coordinates=None))

    def test_jump_proposal_update_eigenvectors_1_d(self):
        coordinates = [proposal.Sample(dict(periodic=0.3)), proposal.Sample(dict(periodic=0.1))]
        with mock.patch('numpy.var') as m:
            m.return_value = 1
            self.jump_proposal.update_eigenvectors(coordinates)
            self.assertTrue(np.equal(np.array([1]), self.jump_proposal.eigen_values))
            self.assertTrue(np.equal(np.array([1]), self.jump_proposal.covariance))
            self.assertTrue(np.equal(np.array([[1.]]), self.jump_proposal.eigen_vectors))

    def test_jump_proposal_update_eigenvectors_n_d(self):
        coordinates = [proposal.Sample(dict(periodic=0.3, reflective=0.3, default=0.3)),
                       proposal.Sample(dict(periodic=0.1, reflective=0.1, default=0.1))]
        with mock.patch('numpy.cov') as m:
            with mock.patch('numpy.linalg.eigh') as n:
                m.side_effect = lambda x: x
                n.return_value = 1, 2
                self.jump_proposal.update_eigenvectors(coordinates)
                self.assertTrue(np.array_equal(np.array([[0.3, 0.1], [0.3, 0.1], [0.3, 0.1]]), self.jump_proposal.covariance))
                self.assertEqual(1, self.jump_proposal.eigen_values)
                self.assertEqual(2, self.jump_proposal.eigen_vectors)

    def test_jump_proposal_call(self):
        self.jump_proposal.update_eigenvectors = lambda x: None
        self.jump_proposal.eigen_values = np.array([1, np.nan, np.nan])
        self.jump_proposal.eigen_vectors = np.array([[0.1, np.nan, np.nan],
                                                    [0.4, np.nan, np.nan],
                                                    [0.7, np.nan, np.nan]])
        with mock.patch('random.randrange') as m:
            with mock.patch('random.gauss') as n:
                m.return_value = 0
                n.return_value = 1
                expected = proposal.Sample()
                expected['periodic'] = 0.2
                expected['reflective'] = 0.5
                expected['default'] = 0.8
                sample = proposal.Sample()
                sample['periodic'] = 0.1
                sample['reflective'] = 0.1
                sample['default'] = 0.1
                new_sample = self.jump_proposal(sample, coordinates=None)
                for key, value in new_sample.items():
                    self.assertAlmostEqual(expected[key], value)


class TestSkyLocationWanderJump(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(ra=prior.Uniform(minimum=0.0, maximum=2*np.pi, boundary='periodic'),
                                           dec=prior.Uniform(minimum=0.0, maximum=np.pi, boundary='reflective')))
        self.jump_proposal = bilby.gw.sampler.proposal.SkyLocationWanderJump(priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_jump_proposal_call_without_inverse_temperature(self):
        with mock.patch('random.gauss') as m:
            m.return_value = 1
            sample = proposal.Sample(dict(ra=0.2, dec=-0.5))
            expected = proposal.Sample(dict(ra=1.2, dec=0.5))
            new_sample = self.jump_proposal(sample)
            for key, value in new_sample.items():
                self.assertAlmostEqual(expected[key], value)
            m.assert_called_with(0, 1.0 / 2 / np.pi)

    def test_jump_proposal_call_with_inverse_temperature(self):
        with mock.patch('random.gauss') as m:
            m.return_value = 1
            sample = proposal.Sample(dict(ra=0.2, dec=-0.5))
            expected = proposal.Sample(dict(ra=1.2, dec=0.5))
            new_sample = self.jump_proposal(sample, inverse_temperature=2.0)
            for key, value in new_sample.items():
                self.assertAlmostEqual(expected[key], value)
            m.assert_called_with(0, np.sqrt(1 / 2.0) / 2 / np.pi)


class TestCorrelatedPolarisationPhaseJump(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(phase=prior.Uniform(minimum=0.0, maximum=2*np.pi),
                                           psi=prior.Uniform(minimum=0.0, maximum=np.pi)))
        self.jump_proposal = bilby.gw.sampler.proposal.CorrelatedPolarisationPhaseJump(priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_jump_proposal_call_case_1(self):
        with mock.patch('random.random') as m:
            m.return_value = 0.3
            sample = proposal.Sample(dict(phase=0.2, psi=0.5))
            alpha = 3.0 * np.pi * 0.3
            beta = 0.3
            expected = proposal.Sample(dict(phase=0.5*(alpha-beta), psi=0.5*(alpha+beta)))
            self.assertEqual(expected, self.jump_proposal(sample, coordinates=None))

    def test_jump_proposal_call_case_2(self):
        with mock.patch('random.random') as m:
            m.return_value = 0.7
            sample = proposal.Sample(dict(phase=0.2, psi=0.5))
            alpha = 0.7
            beta = 3.0 * np.pi * 0.7 - 2 * np.pi
            expected = proposal.Sample(dict(phase=0.5*(alpha-beta), psi=0.5*(alpha+beta)))
            self.assertEqual(expected, self.jump_proposal(sample))


class TestPolarisationPhaseJump(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(phase=prior.Uniform(minimum=0.0, maximum=2*np.pi),
                                           psi=prior.Uniform(minimum=0.0, maximum=np.pi)))
        self.jump_proposal = bilby.gw.sampler.proposal.PolarisationPhaseJump(priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_jump_proposal_call(self):
        sample = proposal.Sample(dict(phase=0.2, psi=0.5))
        expected = proposal.Sample(dict(phase=0.2+np.pi, psi=0.5+np.pi/2))
        self.assertEqual(expected, self.jump_proposal(sample))


class TestDrawFlatPrior(unittest.TestCase):

    def setUp(self):
        self.priors = prior.PriorDict(dict(phase=prior.Uniform(minimum=0.0, maximum=2*np.pi),
                                           psi=prior.Cosine(minimum=0.0, maximum=np.pi)))
        self.jump_proposal = proposal.DrawFlatPrior(priors=self.priors)

    def tearDown(self):
        del self.priors
        del self.jump_proposal

    def test_jump_proposal_call(self):
        with mock.patch('bilby.core.prior.Uniform.sample') as m:
            m.return_value = 0.3
            sample = proposal.Sample(dict(phase=0.2, psi=0.5))
            expected = proposal.Sample(dict(phase=0.3, psi=0.3))
            self.assertEqual(expected, self.jump_proposal(sample))



from __future__ import absolute_import, division

import unittest

from bilby.core import utils


class TestInferParameters(unittest.TestCase):

    def setUp(self):
        def source_function1(freqs, a, b: int):
            return None

        def source_function2(freqs, a, b, *args, **kwargs):
            return None

        def source_function3(freqs, a, b: int, *args, **kwargs):
            return None

        class TestClass:
            def test_method(self, a, b: int, *args, **kwargs):
                pass

        class TestClass2:
            def test_method(self, freqs, a, b: int, *args, **kwargs):
                pass

        self.source1 = source_function1
        self.source2 = source_function2
        self.source3 = source_function3
        test_obj = TestClass()
        self.source4 = test_obj.test_method
        test_obj2 = TestClass()
        self.source5 = test_obj2.test_method

    def tearDown(self):
        del self.source1
        del self.source2
        del self.source3
        del self.source4

    def test_type_hinting(self):
        expected = ['a', 'b']
        actual = utils.infer_parameters_from_function(self.source1)
        self.assertListEqual(expected, actual)

    def test_args_kwargs_handling(self):
        expected = ['a', 'b']
        actual = utils.infer_parameters_from_function(self.source2)
        self.assertListEqual(expected, actual)

    def test_both(self):
        expected = ['a', 'b']
        actual = utils.infer_parameters_from_function(self.source3)
        self.assertListEqual(expected, actual)

    def test_self_handling(self):
        expected = ['a', 'b']
        actual = utils.infer_args_from_method(self.source4)
        self.assertListEqual(expected, actual)

    def test_self_handling_method_as_function(self):
        expected = ['a', 'b']
        actual = utils.infer_args_from_method(self.source5)
        self.assertListEqual(expected, actual)



from __future__ import absolute_import

import os

import numpy as np

import bilby
from bilby.gw.waveform_generator import WaveformGenerator

np.random.seed(10)

time_duration = 4.
sampling_frequency = 4096.

simulation_parameters = dict(
    mass_1=36.,
    mass_2=29.,
    a_1=0.,
    a_2=0.,
    tilt_1=0.,
    tilt_2=0.,
    phi_12=0.,
    phi_jl=0.,
    luminosity_distance=100.,
    theta_jn=0.4,
    phase=1.3,
    waveform_approximant='IMRPhenomPv2',
    reference_frequency=50.,
    ra=1.375,
    dec=-1.2108,
    geocent_time=1126259642.413,
    psi=2.659
)

waveform_generator = WaveformGenerator(
    duration=time_duration, sampling_frequency=sampling_frequency,
    frequency_domain_source_model=bilby.gw.source.lal_binary_black_hole,
    parameters=simulation_parameters)

signal = waveform_generator.frequency_domain_strain()

IFO = bilby.gw.detector.get_interferometer_with_fake_noise_and_injection(
    name='H1', injection_polarizations=signal,
    injection_parameters=simulation_parameters, duration=time_duration,
    plot=False, sampling_frequency=sampling_frequency)

hf_signal_and_noise = IFO.strain_data.frequency_domain_strain
frequencies = bilby.core.utils.create_frequency_series(
    sampling_frequency=sampling_frequency, duration=time_duration)

if __name__ == '__main__':
    dir_path = os.path.dirname(os.path.realpath(__file__))
    with open(dir_path + '/standard_data.txt', 'w+') as f:
        np.savetxt(
            f,
            np.column_stack([frequencies,
                             hf_signal_and_noise.view(float).reshape(-1, 2)]),
            header='frequency hf_real hf_imag')



from __future__ import division, absolute_import
import unittest

import numpy as np
from copy import copy

import bilby


class TestLalBBH(unittest.TestCase):

    def setUp(self):
        self.parameters = dict(
            mass_1=30.0, mass_2=30.0, luminosity_distance=400.0, a_1=0.4,
            tilt_1=0.2, phi_12=1.0, a_2=0.8, tilt_2=2.7, phi_jl=2.9,
            theta_jn=0.3, phase=0.0)
        self.waveform_kwargs = dict(
            waveform_approximant='IMRPhenomPv2', reference_frequency=50.0,
            minimum_frequency=20.0, catch_waveform_errors=True)
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)
        self.bad_parameters = copy(self.parameters)
        self.bad_parameters['mass_1'] = -30.
    
    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array
        del self.bad_parameters

    def test_lal_bbh_works_runs_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby.gw.source.lal_binary_black_hole(
                self.frequency_array, **self.parameters), dict)

    def test_waveform_error_catching(self):
        self.bad_parameters.update(self.waveform_kwargs)
        self.assertIsNone(bilby.gw.source.lal_binary_black_hole(
            self.frequency_array, **self.bad_parameters))

    def test_waveform_error_raising(self):
        raise_error_parameters = copy(self.bad_parameters)
        raise_error_parameters.update(self.waveform_kwargs)
        raise_error_parameters['catch_waveform_errors']=False
        with self.assertRaises(Exception):
            bilby.gw.source.lal_binary_black_hole(
                self.frequency_array, **raise_error_parameters)

    def test_lal_bbh_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby.gw.source.lal_binary_black_hole(
                self.frequency_array, **self.parameters), dict)

    # Removed due to issue with SimInspiralFD - see https://git.ligo.org/lscsoft/lalsuite/issues/153
    # def test_lal_bbh_works_with_time_domain_approximant(self):
    #     self.waveform_kwargs['waveform_approximant'] = 'SEOBNRv3'
    #     self.parameters.update(self.waveform_kwargs)
    #     self.assertIsInstance(
    #         bilby.gw.source.lal_binary_black_hole(
    #             self.frequency_array, **self.parameters), dict)


class TestLalBNS(unittest.TestCase):

    def setUp(self):
        self.parameters = dict(
            mass_1=1.4, mass_2=1.4, luminosity_distance=400.0, a_1=0.4,
            a_2=0.3, tilt_1=0.2, tilt_2=1.7, phi_jl=0.2, phi_12=0.9,
            theta_jn=1.7, phase=0.0, lambda_1=100.0, lambda_2=100.0)
        self.waveform_kwargs = dict(
            waveform_approximant='IMRPhenomPv2_NRTidal',
            reference_frequency=50.0, minimum_frequency=20.0)
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array

    def test_lal_bns_runs_with_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby.gw.source.lal_binary_neutron_star(
                self.frequency_array, **self.parameters), dict)

    def test_lal_bns_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby.gw.source.lal_binary_neutron_star(
                self.frequency_array, **self.parameters), dict)

    def test_fails_without_tidal_parameters(self):
        self.parameters.pop('lambda_1')
        self.parameters.pop('lambda_2')
        self.parameters.update(self.waveform_kwargs)
        with self.assertRaises(TypeError):
            bilby.gw.source.lal_binary_neutron_star(
                self.frequency_array, **self.parameters)


class TestEccentricLalBBH(unittest.TestCase):

    def setUp(self):
        self.parameters = dict(
            mass_1=30.0, mass_2=30.0, luminosity_distance=400.0, theta_jn=0.0,
            phase=0.0, eccentricity=0.1)
        self.waveform_kwargs = dict(
            waveform_approximant='EccentricFD', reference_frequency=10.0,
            minimum_frequency=10.0)
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array

    def test_lal_ebbh_works_runs_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby.gw.source.lal_eccentric_binary_black_hole_no_spins(
                self.frequency_array, **self.parameters), dict)

    def test_lal_ebbh_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby.gw.source.lal_eccentric_binary_black_hole_no_spins(
                self.frequency_array, **self.parameters), dict)

    def test_fails_without_eccentricity(self):
        self.parameters.pop('eccentricity')
        self.parameters.update(self.waveform_kwargs)
        with self.assertRaises(TypeError):
            bilby.gw.source.lal_eccentric_binary_black_hole_no_spins(
                self.frequency_array, **self.parameters)


class TestROQBBH(unittest.TestCase):

    def setUp(self):
        roq_dir = '/roq_basis'

        fnodes_linear_file = "{}/fnodes_linear.npy".format(roq_dir)
        fnodes_linear = np.load(fnodes_linear_file).T
        fnodes_quadratic_file = "{}/fnodes_quadratic.npy".format(roq_dir)
        fnodes_quadratic = np.load(fnodes_quadratic_file).T

        self.parameters = dict(
            mass_1=30.0, mass_2=30.0, luminosity_distance=400.0, a_1=0.0,
            tilt_1=0.0, phi_12=0.0, a_2=0.0, tilt_2=0.0, phi_jl=0.0,
            theta_jn=0.0, phase=0.0)
        self.waveform_kwargs = dict(
            frequency_nodes_linear=fnodes_linear,
            frequency_nodes_quadratic=fnodes_quadratic,
            reference_frequency=50., minimum_frequency=20.,
            approximant='IMRPhenomPv2')
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array

    def test_roq_runs_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby.gw.source.roq(self.frequency_array, **self.parameters), dict)

    def test_roq_fails_without_frequency_nodes(self):
        self.parameters.update(self.waveform_kwargs)
        del self.parameters['frequency_nodes_linear']
        del self.parameters['frequency_nodes_quadratic']
        with self.assertRaises(KeyError):
            bilby.gw.source.roq(self.frequency_array, **self.parameters)


if __name__ == '__main__':
    unittest.main()



from bilby.gw import calibration
import unittest
import numpy as np


class TestBaseClass(unittest.TestCase):

    def setUp(self):
        self.model = calibration.Recalibrate()

    def tearDown(self):
        del self.model

    def test_repr(self):
        expected = 'Recalibrate(prefix={})'.format('\'recalib_\'')
        actual = repr(self.model)
        self.assertEqual(expected, actual)

    def test_calibration_factor(self):
        frequency_array = np.linspace(20, 1024, 1000)
        cal_factor = self.model.get_calibration_factor(frequency_array)
        assert np.alltrue(cal_factor.real == np.ones_like(frequency_array))


class TestCubicSpline(unittest.TestCase):

    def setUp(self):
        self.prefix = 'recalib_'
        self.minimum_frequency = 20
        self.maximum_frequency = 1024
        self.n_points = 5
        self.model = calibration.CubicSpline(
            prefix=self.prefix, minimum_frequency=self.minimum_frequency,
            maximum_frequency=self.maximum_frequency, n_points=self.n_points)
        self.parameters = {'recalib_{}_{}'.format(param, ii): 0.0
                           for ii in range(5)
                           for param in ['amplitude', 'phase']}

    def tearDown(self):
        del self.prefix
        del self.minimum_frequency
        del self.maximum_frequency
        del self.n_points
        del self.model
        del self.parameters

    def test_calibration_factor(self):
        frequency_array = np.linspace(20, 1024, 1000)
        cal_factor = self.model.get_calibration_factor(frequency_array,
                                                       **self.parameters)
        assert np.alltrue(cal_factor.real == np.ones_like(frequency_array))

    def test_repr(self):
        expected = 'CubicSpline(prefix=\'{}\', minimum_frequency={}, maximum_frequency={}, n_points={})'\
            .format(self.prefix, self.minimum_frequency, self.maximum_frequency, self.n_points)
        actual = repr(self.model)
        self.assertEqual(expected, actual)


class TestCubicSplineRequiresFourNodes(unittest.TestCase):

    def test_cannot_instantiate_with_too_few_nodes(self):
        for ii in range(6):
            if ii < 4:
                with self.assertRaises(ValueError):
                    calibration.CubicSpline('test', 1, 10, ii)
            else:
                calibration.CubicSpline('test', 1, 10, ii)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
import matplotlib
matplotlib.use('Agg')

import unittest
import os
import shutil
import logging

# Required to run the tests
from past.builtins import execfile
import bilby.core.utils

# Imported to ensure the examples run
import numpy as np
import inspect

bilby.core.utils.command_line_args.bilby_test_mode = True


class Test(unittest.TestCase):
    outdir = 'outdir'
    dir_path = os.path.dirname(os.path.realpath(__file__))
    dir_path = os.path.abspath(os.path.join(dir_path, os.path.pardir))

    @classmethod
    def setUpClass(self):
        if os.path.isdir(self.outdir):
            try:
                shutil.rmtree(self.outdir)
            except OSError:
                logging.warning(
                    "{} not removed prior to tests".format(self.outdir))

    @classmethod
    def tearDownClass(self):
        if os.path.isdir(self.outdir):
            try:
                shutil.rmtree(self.outdir)
            except OSError:
                logging.warning(
                    "{} not removed prior to tests".format(self.outdir))

    def test_examples(self):
        """ Loop over examples to check they run """
        examples = ['examples/gw_examples/injection_examples/fast_tutorial.py',
                    'examples/gw_examples/data_examples/GW150914.py',
                    ]
        for filename in examples:
            print("Testing {}".format(filename))
            execfile(filename)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division

import unittest
import os
from shutil import rmtree

import numpy as np
import gwpy
import lal
import lalsimulation as lalsim

import bilby
from bilby.gw import utils as gwutils


class TestGWUtils(unittest.TestCase):

    def setUp(self):
        self.outdir = 'outdir'
        bilby.core.utils.check_directory_exists_and_if_not_mkdir(self.outdir)

    def tearDown(self):
        try:
            rmtree(self.outdir)
        except FileNotFoundError:
            pass

    def test_asd_from_freq_series(self):
        freq_data = np.array([1, 2, 3])
        df = 0.1
        asd = gwutils.asd_from_freq_series(freq_data, df)
        self.assertTrue(np.all(asd == freq_data * 2 * df ** 0.5))

    def test_psd_from_freq_series(self):
        freq_data = np.array([1, 2, 3])
        df = 0.1
        psd = gwutils.psd_from_freq_series(freq_data, df)
        self.assertTrue(np.all(psd == (freq_data * 2 * df ** 0.5)**2))

    def test_time_delay_from_geocenter(self):
        """
        The difference in the two detector case is due to rounding error.
        Different hardware gives different numbers in the last decimal place.
        """
        det1 = np.array([0.1, 0.2, 0.3])
        det2 = np.array([0.1, 0.2, 0.5])
        ra = 0.5
        dec = 0.2
        time = 10
        self.assertEqual(
            gwutils.time_delay_geocentric(det1, det1, ra, dec, time), 0)
        self.assertAlmostEqual(
            gwutils.time_delay_geocentric(det1, det2, ra, dec, time),
            1.3253791114055397e-10, 14)

    def test_get_polarization_tensor(self):
        ra = 1
        dec = 2.0
        time = 10
        psi = 0.1
        for mode in ['plus', 'cross', 'breathing', 'longitudinal', 'x', 'y']:
            p = gwutils.get_polarization_tensor(ra, dec, time, psi, mode)
            self.assertEqual(p.shape, (3, 3))
        with self.assertRaises(ValueError):
            gwutils.get_polarization_tensor(ra, dec, time, psi, 'not-a-mode')

    def test_inner_product(self):
        aa = np.array([1, 2, 3])
        bb = np.array([5, 6, 7])
        frequency = np.array([0.2, 0.4, 0.6])
        PSD = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        ip = gwutils.inner_product(aa, bb, frequency, PSD)
        self.assertEqual(ip, 0)

    def test_noise_weighted_inner_product(self):
        aa = np.array([1e-23, 2e-23, 3e-23])
        bb = np.array([5e-23, 6e-23, 7e-23])
        frequency = np.array([100, 101, 102])
        PSD = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        psd = PSD.power_spectral_density_interpolated(frequency)
        duration = 4
        nwip = gwutils.noise_weighted_inner_product(aa, bb, psd, duration)
        self.assertEqual(nwip, 239.87768033598326)

        self.assertEqual(
            gwutils.optimal_snr_squared(aa, psd, duration),
            gwutils.noise_weighted_inner_product(aa, aa, psd, duration))

    def test_matched_filter_snr(self):
        signal = np.array([1e-23, 2e-23, 3e-23])
        frequency_domain_strain = np.array([5e-23, 6e-23, 7e-23])
        frequency = np.array([100, 101, 102])
        PSD = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        psd = PSD.power_spectral_density_interpolated(frequency)
        duration = 4

        mfsnr = gwutils.matched_filter_snr(
            signal, frequency_domain_strain, psd, duration)
        self.assertEqual(mfsnr, 25.510869054168282)

    def test_get_event_time(self):
        events = ['GW150914', 'GW151012', 'GW151226', 'GW170104', 'GW170608',
                  'GW170729', 'GW170809', 'GW170814', 'GW170817', 'GW170818',
                  'GW170823']
        for event in events:
            self.assertTrue(isinstance(gwutils.get_event_time(event), float))

        self.assertTrue(gwutils.get_event_time('GW010290') is None)

    def test_read_frame_file(self):
        start_time = 0
        end_time = 10
        channel = 'H1:GDS-CALIB_STRAIN'
        N = 100
        times = np.linspace(start_time, end_time, N)
        data = np.random.normal(0, 1, N)
        ts = gwpy.timeseries.TimeSeries(data=data, times=times, t0=0)
        ts.channel = gwpy.detector.Channel(channel)
        ts.name = channel
        filename = os.path.join(self.outdir, 'test.gwf')
        ts.write(filename, format='gwf')

        # Check reading without time limits
        strain = gwutils.read_frame_file(
            filename, start_time=None, end_time=None, channel=channel)
        self.assertEqual(strain.channel.name, channel)
        self.assertTrue(np.all(strain.value==data[:-1]))

        # Check reading with time limits
        start_cut = 2
        end_cut = 8
        strain = gwutils.read_frame_file(
            filename, start_time=start_cut, end_time=end_cut, channel=channel)
        idxs = (times > start_cut) & (times < end_cut)
        # Dropping the last element - for some reason gwpy drops the last element when reading in data
        self.assertTrue(np.all(strain.value==data[idxs][:-1]))

        # Check reading with unknown channels
        strain = gwutils.read_frame_file(
            filename, start_time=None, end_time=None)
        self.assertTrue(np.all(strain.value==data[:-1]))

        # Check reading with incorrect channel
        strain = gwutils.read_frame_file(
            filename, start_time=None, end_time=None, channel='WRONG')
        self.assertTrue(np.all(strain.value==data[:-1]))

        ts = gwpy.timeseries.TimeSeries(data=data, times=times, t0=0)
        ts.name = 'NOT-A-KNOWN-CHANNEL'
        ts.write(filename, format='gwf')
        strain = gwutils.read_frame_file(
            filename, start_time=None, end_time=None)
        self.assertEqual(strain, None)

    def test_convert_args_list_to_float(self):
        self.assertEqual(
            gwutils.convert_args_list_to_float(1, '2', 3.0), [1.0, 2.0, 3.0])
        with self.assertRaises(ValueError):
            gwutils.convert_args_list_to_float(1, '2', 'ten')

    def test_lalsim_SimInspiralTransformPrecessingNewInitialConditions(self):
        a = gwutils.lalsim_SimInspiralTransformPrecessingNewInitialConditions( 
            0.1, 0, 0.6, 0.5, 0.6, 0.1, 0.8, 30.6, 23.2, 50, 0)
        self.assertTrue(len(a) == 7)

    def test_get_approximant(self):
        with self.assertRaises(ValueError):
            gwutils.lalsim_GetApproximantFromString(10)

    def test_lalsim_SimInspiralChooseFDWaveform(self):
        a = gwutils.lalsim_SimInspiralChooseFDWaveform(
            35.2, 20.4, 0.1, 0.2, 0.2, 0.2, 0.2, 0.1, 1000, 2, 2.3,
            45., 0.1, 10, 0.01, 10, 1000, 20, None, lalsim.IMRPhenomPv2)
        self.assertEqual(len(a), 2)
        self.assertEqual(type(a[0]), lal.COMPLEX16FrequencySeries)
        self.assertEqual(type(a[1]), lal.COMPLEX16FrequencySeries)

        with self.assertRaises(RuntimeError):
            _ = gwutils.lalsim_SimInspiralChooseFDWaveform(
                35.2, 20.4, 0.1, 0.2, 0.2, 0.2, 0.2, 0.1, 1000, 2, 2.3,
                45., 0.1, 10, 0.01, 10, 1000, 20, None, 'Fail')

        with self.assertRaises(ValueError):
            _ = gwutils.lalsim_SimInspiralChooseFDWaveform(
                35.2, 20.4, 0.1, 0.2, 0.2, 0.2, 0.2, 0.1, 1000, 2, 2.3,
                45., 0.1, 10, 0.01, 10, 1000, 20, None, 1.5)


if __name__ == '__main__':
    unittest.main()
